// code by jph
package kah;

import java.util.Map;
import java.util.Map.Entry;
import java.util.TreeMap;

import kah.mov.Move;
import kah.mov.UserMove;

public abstract class Minimax {
  final UserMove myUserRoot;
  public Result myResult;

  public Minimax(UserMove myUserRoot) {
    this.myUserRoot = myUserRoot;
  }

  protected Result computeResult() {
    if (myUserRoot.map.isEmpty())
      return null;
    Map<Integer, Integer> map = new TreeMap<>();
    for (Entry<Integer, Move> entry : myUserRoot.map.entrySet())
      map.put(entry.getKey(), entry.getValue().minimax());
    return new Result(map);
  }
  
  abstract int getLeafCount();

  public String printInfo() {
    int nodes = 1000; // FIXME
    int leafs = getLeafCount();
    int total_depth = getLeafDepth(myUserRoot);
    double ratio = total_depth / (double) leafs;
    // System.out.println("------------------");
    // System.out.println("nodes: " + nodes);
    // System.out.println("leafs: " + leafs);
    // System.out.println("depth: " + ratio);
    return String.format("nodes=%d leafs=%d depth=%6.3f", nodes, leafs, ratio);
  }

  private int getLeafDepth(Move move) {
    if (move.isLeaf())
      return move.depth;
    int sum = 0;
    for (Entry<Integer, Move> entry : move.map.entrySet())
      sum += getLeafDepth(entry.getValue());
    return sum;
  }
}
