// code by jph
package kah;

import java.util.ArrayList;
import java.util.Collections;
import java.util.List;
import java.util.Map;
import java.util.Map.Entry;
import java.util.Random;

/**
 * instance of Minimax represents outcome of board analysis.
 * the field map contains the evaluation of possible moves. 
 */
public class Result {
  private static final Random RANDOM = new Random();
  // ---
  public final Map<Integer, Integer> map;
  public final int pos;
  public final int value;

  Result(Map<Integer, Integer> map) {
    assert !map.isEmpty();
    this.map = Collections.unmodifiableMap(map);
    pos = private_pick();
    value = this.map.get(pos);
  }

  private int private_pick() {
    int max = Integer.MIN_VALUE; // track minimum
    List<Integer> list = new ArrayList<>();
    for (Entry<Integer, Integer> entry : map.entrySet()) {
      int cmp = entry.getValue();
      if (max == cmp)
        list.add(entry.getKey());
      if (max < cmp) {
        list.clear();
        list.add(entry.getKey());
        max = cmp;
      }
    }
    return list.get(RANDOM.nextInt(list.size()));
  }

}
