// code by jph
package kah.gui;

import java.awt.Color;
import java.awt.Font;
import java.awt.FontMetrics;
import java.awt.Graphics;
import java.awt.Graphics2D;
import java.awt.Point;
import java.awt.Shape;
import java.awt.event.MouseEvent;
import java.awt.geom.Ellipse2D;
import java.awt.geom.Rectangle2D;
import java.util.HashMap;
import java.util.Map;
import java.util.Map.Entry;

import javax.swing.JComponent;

import kah.Board;
import kah.DepthFirstMinimax;
import kah.Kalaha;
import kah.Minimax;
import kah.Result;
import sys.IntRange;
import sys.gui.GraphicsUtil;
import sys.gui.LazyMouse;
import sys.gui.LazyMouseListener;

public class KalahaComponent {
  private Kalaha kalaha;
  int flips = 6;
  Minimax self_minimax = null;
  Result user_minimax = null;
  private boolean displayHint = false;

  void setKalaha(Kalaha kalaha) {
    this.kalaha = kalaha;
    self_minimax = null;
    myJComponent.repaint();
  }

  void setDisplayHint(boolean value) {
    displayHint = value;
    myJComponent.repaint();
  }

  final double ys = 50;
  final double w = 50;

  double getX_of(int pos) {
    return 100 + w * 1.5 * pos;
  }

  private Map<Shape, Integer> myUserShapes = new HashMap<>();
  JComponent myJComponent = new JComponent() {
    @Override
    protected void paintComponent(Graphics graphics) {
      Board board = kalaha.board_display;
      final Graphics2D myGraphics = (Graphics2D) graphics;
      GraphicsUtil.setQualityHigh(myGraphics);
      myUserShapes.clear();
      {
        // draw green turn indicator
        double y = kalaha.user_display ? 150 : 50;
        Shape shape = new Rectangle2D.Double(100, y, 5 * 1.5 * w + w, w);
        myGraphics.setColor(Color.green);
        myGraphics.fill(shape);
      }
      {
        // draw results of computer analysis
        if (kalaha.user_display) {
          if (displayHint) {
            user_minimax = new DepthFirstMinimax(kalaha.board_display, 4).myResult; // TODO not final design
            if (user_minimax != null) {
              myGraphics.setColor(Color.gray);
              myGraphics.setFont(new Font(Font.DIALOG, Font.BOLD, 11));
              final double y = 200 + 13;
              for (Entry<Integer, Integer> entry : user_minimax.map.entrySet()) {
                double x = getX_of(entry.getKey());
                myGraphics.drawString("" + entry.getValue(), (int) x, (int) y);
              }
            }
          }
        } else if (self_minimax != null) {
          {
            myGraphics.setColor(Color.black);
            myGraphics.setFont(new Font(Font.DIALOG, Font.PLAIN, 11));
            myGraphics.drawString(self_minimax.printInfo(), 0, 10);
          }
          {
            double x = getX_of(5 - self_minimax.myResult.pos);
            Shape shape = new Rectangle2D.Double(x, ys, w, w);
            myGraphics.setColor(Color.yellow);
            myGraphics.fill(shape);
          }
          myGraphics.setColor(Color.gray);
          myGraphics.setFont(new Font(Font.DIALOG, Font.BOLD, 11));
          final double y = 100 + 13;
          for (Entry<Integer, Integer> entry : self_minimax.myResult.map.entrySet()) {
            double x = getX_of(5 - entry.getKey());
            myGraphics.drawString("" + entry.getValue(), (int) x, (int) y);
          }
        }
      }
      // draw holes and number of stones
      myGraphics.setFont(new Font(Font.DIALOG, Font.BOLD, 20));
      final FontMetrics myFontMetrics = myGraphics.getFontMetrics();
      for (int base : new IntRange(6)) {
        {
          int index = base;
          double x = getX_of(base);
          double y = 150;
          Shape shape = new Ellipse2D.Double(x, y, w, w);
          myUserShapes.put(shape, index);
          // ---
          myGraphics.setColor(Color.red);
          myGraphics.fill(shape);
          myGraphics.setColor(Color.white);
          final String myString = "" + board.get(index);
          int width = myFontMetrics.stringWidth(myString);
          myGraphics.drawString(myString, (int) (x + w / 2 - width / 2), (int) (y + w * 2 / 3));
        }
        {
          int index = 7 + base;
          double x = getX_of(5 - base);
          Shape shape = new Ellipse2D.Double(x, ys, w, w);
          // ---
          myGraphics.setColor(Color.blue);
          myGraphics.fill(shape);
          myGraphics.setColor(Color.white);
          final String myString = "" + board.get(index);
          int width = myFontMetrics.stringWidth(myString);
          myGraphics.drawString(myString, (int) (x + w / 2 - width / 2), (int) (ys + w * 2 / 3));
        }
      }
      {
        // draw number of stones in Kalahas
        final double y = 100;
        {
          double x = getX_of(6);
          Shape shape = new Ellipse2D.Double(x, y, w, w);
          myGraphics.setColor(Color.red);
          myGraphics.draw(shape);
          final String myString = "" + board.get(6);
          int width = myFontMetrics.stringWidth(myString);
          myGraphics.setColor(Color.black);
          myGraphics.drawString(myString, (int) (x + w / 2 - width / 2), (int) (y + w * 2 / 3));
        }
        // ---
        {
          double x = getX_of(-1);
          Shape shape = new Ellipse2D.Double(x, y, w, w);
          myGraphics.setColor(Color.blue);
          myGraphics.draw(shape);
          final String myString = "" + board.get(13);
          int width = myFontMetrics.stringWidth(myString);
          myGraphics.setColor(Color.black);
          myGraphics.drawString(myString, (int) (x + w / 2 - width / 2), (int) (y + w * 2 / 3));
        }
      }
    };
  };

  public KalahaComponent() {
    LazyMouseListener myLazyMouseListener = new LazyMouseListener() {
      @Override
      public void lazyClicked(MouseEvent myMouseEvent) {
        if (kalaha.user_display) {
          Point point = myMouseEvent.getPoint();
          for (Entry<Shape, Integer> entry : myUserShapes.entrySet()) {
            Shape shape = entry.getKey();
            if (shape.contains(point)) {
              int pos = entry.getValue();
              kalaha.userSelect(pos);
            }
          }
        } else
          kalaha.advance();
        // ---
        if (!kalaha.isUser())
          self_minimax = kalaha.announceSelf(flips);
        // ---
        myJComponent.repaint();
      }
    };
    myJComponent.addMouseListener(new LazyMouse(myLazyMouseListener));
  }
}
