// code by jph
package kah.mov;

import java.util.HashMap;
import java.util.Map;

import kah.Board;

/**
 * Move encodes
 *  + single selection of pit, or terminal move.
 *  + side of turn: {@link SelfMove} or {@link UserMove}
 */
public abstract class Move {
  /**
   * standard moves are indexed by the pit number 0, 1,..., 5
   * the terminal move is indexed with 6
   */
  public static final int TERMINAL_KEY = 6;
  public final Board board;
  public final int depth;
  public final int flips;
  /**
   * map of children moves
   */
  public Map<Integer, Move> map = new HashMap<>();

  protected Move(Board board, int depth, int flips) {
    this.board = board;
    this.depth = depth;
    this.flips = flips;
  }

  protected final Move protected_child(int pos, Move move) {
    map.put(pos, move);
    return move;
  }

  public final boolean isLeaf() {
    return map.isEmpty();
  }

  /**
   * @return maximum loss (for subsequent minimization)
   */
  public abstract int minimax();

  /**
   * create move and add as child to this
   * 
   * @param pos in 0, 1, ..., 5
   * @return
   */
  public abstract Move nextMove(int pos);

  /**
   * create terminal move
   * 
   * @return
   */
  public abstract Move nextMoveTerminal();
}
